'------------------------------------------------------------------------------
'
'  WSOCK32.INC -- Windows Sockets 1.1 API Declarations for PowerBASIC
'                 plus Internet Protocol Helper functions
'
'  Copyright (c) 1997-2002 PowerBASIC, Inc.
'  Portions Copyright (c) 1993-1999 Microsoft Corporation
'  Portions Copyright (c) 1982-1986 Regents of the University of California
'  All Rights Reserved.
'
'  Last Update: 30 October 2002
'
'  This file contains the constants, TYPE, and DECLARE statements for the
'  32-bit Windows Sockets 1.1 API (winsock.h of 23 May 2001), plus several
'  "mystery meat" ICMP* functions (said to be from unavailable icmpapi.h).
'
'------------------------------------------------------------------------------

#IF NOT %DEF(%WSOCK32_INC)

%WSOCK32_INC = 1

#IF NOT %DEF(%WINAPI)
    #INCLUDE "WIN32API.INC"
#ENDIF

' Select uses arrays of SOCKETs (DWORDs).  These macros manipulate such
' arrays. %FD_SETSIZE may be defined by the user before including
' this file, but the default here should be >= 64.

#IF NOT %DEF(%FD_SETSIZE)
    %FD_SETSIZE = 64
#ENDIF

TYPE fd_setstruc   ' fd_set in winsock.h renamed here to avoid conflicts
    fd_count AS DWORD                         ' how many are SET?
    fd_array (0 TO %FD_SETSIZE - 1) AS DWORD  ' an array of SOCKETs
END TYPE

DECLARE FUNCTION WSAFDIsSet LIB "wsock32.dll" ALIAS "__WSAFDIsSet" (BYVAL ssocket AS DWORD, xfd AS fd_setstruc) AS LONG

#IF %DEF(%USEMACROS) '---------------------------------------------------------

    MACRO FD_CLR (xfd, xset)
        MACROTEMP ix
        ix = 0
        DO WHILE ix < @xset.fd_count
            IF @xset.fd_array( ix ) = xfd THEN
                DO WHILE ix < @xset.fd_count - 1
                    @xset.fd_array( ix ) = @xset.fd_array( ix + 1)
                    INCR ix
                LOOP
                DECR @xset.fd_count
                EXIT DO
            END IF
            INCR ix
        LOOP
    END MACRO

    MACRO FD_SET (xfd, xset)
        IF @xset.fd_count < %FD_SETSIZE THEN
            @xset.fd_array(@xset.fd_count) = xfd
            INCR @xset.fd_count
        END IF
    END MACRO

    MACRO FD_ZERO (xset)
        @xset.fd_count = 0
    END MACRO

    MACRO FD_ISSET (xfd, xset) = WSAFDIsSet(xfd, xset)

#ELSE  ' #IF %DEF(%USEMACROS) --------------------------------------------

    SUB FD_SET (BYVAL s AS LONG, fds AS fd_setstruc)
        IF fds.fd_count < %FD_SETSIZE THEN
            fds.fd_array(fds.fd_count) = s
            INCR fds.fd_count
        END IF
    END SUB

    SUB FD_ZERO (fd AS fd_setstruc)
        fd.fd_count = 0
    END SUB

#ENDIF  ' #IF %DEF(%USEMACROS) ------------------------------------------------

' Structure used in select() call, taken from the BSD file sys/time.h.

TYPE timeval
    tv_sec  AS LONG    ' seconds
    tv_usec AS LONG    ' and microseconds
END TYPE

' Operations on timevals.

#IF %DEF(%USEMACROS)

    MACRO timerisset (tvp) = (tvp.tv_sec OR tvp.tv_usec)

    MACRO timerclear (tvp)
        tvp.tv_sec = 0
        tvp.tv_usec = 0
    END MACRO

#ENDIF  ' #IF %DEF(%USEMACROS)

' Commands for ioctlsocket(),  taken from the BSD file fcntl.h.
'
'
' Ioctl's have the command encoded in the lower word,
' and the size of any in or out parameters in the upper
' word.  The high 2 bits of the upper word are used
' to encode the in/out status of the parameter; for now
' we restrict parameters to at most 128 bytes.

%IOCPARM_MASK = &H7F                  ' parameters must be < 128 bytes
%IOC_VOID     = &H20000000???         ' no parameters
%IOC_OUT      = &H40000000???         ' copy out parameters
%IOC_IN       = &H80000000???         ' copy in parameters
%IOC_INOUT    = (%IOC_IN OR %IOC_OUT) ' &H20000000 distinguishes new &
                                      ' old ioctl's

#IF %DEF(%USEMACROS) '---------------------------------------------------------

    MACRO FUNCTION IO (x, y)
        MACROTEMP t
        LOCAL t AS DWORD
        t = x
        SHIFT LEFT t, 8
    END MACRO = (%IOC_VOID OR t OR y)

    MACRO FUNCTION IOR (x, y, t)
        MACROTEMP t1, t2
        LOCAL t1 AS DWORD
        LOCAL t2 AS DWORD
        t1 = SIZEOF(t) AND %IOCPARM_MASK
        SHIFT LEFT t1, 16
        t2 = x
        SHIFT LEFT t2, 8
    END MACRO = (%IOC_OUT OR t1 OR t2 OR y)

    MACRO FUNCTION IOW (x, y, t)
        MACROTEMP t1, t2
        LOCAL t1 AS DWORD
        LOCAL t2 AS DWORD
        t1 = SIZEOF(t) AND %IOCPARM_MASK
        SHIFT LEFT t1, 16
        t2 = x
        SHIFT LEFT t2, 8
    END MACRO = (%IOC_IN OR t1 OR t2 OR y)

#ENDIF  ' #IF %DEF(%USEMACROS) ------------------------------------------------

%FIONREAD = &H4004667F???  ' get # bytes to read
%FIONBIO  = &H8004667E???  ' set/clear non-blocking i/o
%FIONSYNC = &H8004667D???  ' set/clear async i/o

' Socket I/O Controls
%SIOCSHIWAT = &H80047300???  ' set high watermark
%SIOCGHIWAT = &H80047301???  ' get high watermark
%SIOCSLOWAT = &H80047302???  ' set low watermark
%SIOCGLOWAT = &H80047303???  ' get low watermark
%SIOCATMARK = &H80047307???  ' at oob mark?

' Structures returned by network data base library, taken from the
' BSD file netdb.h.  All addresses are supplied in host order, and
' returned in network order (suitable for use in system calls).

TYPE hostentStru  ' hostent type in winsock.h - renamed to avoid conflicts
    h_name  AS ASCIIZ PTR   ' official name of host
    h_alias AS DWORD        ' alias list (pointer to ASCIIZ PTR list)
    h_addr  AS INTEGER      ' host address type
    h_len   AS INTEGER      ' length of address
    h_list  AS long ptr        ' list of addresses (pointer to ASCIIZ PTR list)
END TYPE

' It is assumed here that a network number
' fits in 32 bits.

TYPE netent
    n_name     AS ASCIIZ PTR  ' official name of net
    n_aliases  AS DWORD       ' alias list (pointer to ASCIIZ PTR list)
    n_addrtype AS INTEGER     ' net address type
    n_net      AS DWORD       ' network #
END TYPE

TYPE servent
    s_name    AS ASCIIZ PTR   ' official service name
    s_aliases AS DWORD        ' alias list (pointer to ASCIIZ PTR list)
    s_port    AS INTEGER      ' port #
    s_proto   AS ASCIIZ PTR   ' protocol to use
END TYPE

TYPE protoent
    p_name AS ASCIIZ PTR      ' official protocol name
    p_aliases AS DWORD        ' alias list (pointer to ASCIIZ PTR list)
    p_proto AS INTEGER        ' protocol #
END TYPE

' WinSock 2 extension -- manifest constants for shutdown()

%SD_RECEIVE = &H00
%SD_SEND    = &H01
%SD_BOTH    = &H02


' Constants and structures defined by the internet system,
' Per RFC 790, September 1981, taken from the BSD file netinet/in.h.

' Protocols
%IPPROTO_IP   = 0                ' dummy for IP
%IPPROTO_ICMP = 1                ' control message protocol
%IPPROTO_GGP  = 2                ' gateway^2 (deprecated)
%IPPROTO_TCP  = 6                ' tcp
%IPPROTO_PUP  = 12               ' pup
%IPPROTO_UDP  = 17               ' user datagram protocol
%IPPROTO_IDP  = 22               ' xns idp
%IPPROTO_ND   = 77               ' UNOFFICIAL net disk proto

%IPPROTO_RAW  = 255              ' raw IP packet
%IPPROTO_MAX  = 256

' Port/socket numbers: network standard functions
%IPPORT_ECHO        = 7
%IPPORT_DISCARD     = 9
%IPPORT_SYSTAT      = 11
%IPPORT_DAYTIME     = 13
%IPPORT_NETSTAT     = 15
%IPPORT_FTP         = 21
%IPPORT_TELNET      = 23
%IPPORT_SMTP        = 25
%IPPORT_TIMESERVER  = 37
%IPPORT_NAMESERVER  = 42
%IPPORT_WHOIS       = 43
%IPPORT_MTP         = 57

' Port/socket numbers: host specific functions

%IPPORT_TFTP        = 69
%IPPORT_RJE         = 77
%IPPORT_FINGER      = 79
%IPPORT_TTYLINK     = 87
%IPPORT_SUPDUP      = 95

' UNIX TCP sockets

%IPPORT_EXECSERVER  = 512
%IPPORT_LOGINSERVER = 513
%IPPORT_CMDSERVER   = 514
%IPPORT_EFSSERVER   = 520

' UNIX UDP sockets

%IPPORT_BIFFUDP     = 512
%IPPORT_WHOSERVER   = 513
%IPPORT_ROUTESERVER = 520
                    ' 520+1 also used

' Ports < IPPORT_RESERVED are reserved for
' privileged processes (e.g. root).

%IPPORT_RESERVED    = 1024

' Link numbers

%IMPLINK_IP        = 155
%IMPLINK_LOWEXPER  = 156
%IMPLINK_HIGHEXPER = 158

' Internet address (old style... should be updated)

TYPE S_un_b_type
    s_b1 AS BYTE
    s_b2 AS BYTE
    s_b3 AS BYTE
    s_b4 AS BYTE
END TYPE

TYPE S_un_w_type
    s_w1 AS WORD
    s_w2 AS WORD
END TYPE

UNION S_un
    S_un_b AS S_un_b_type
    S_un_w AS S_un_w_type
    S_addr AS DWORD
END UNION

UNION in_addr
    S_un_b AS S_un_b_type
    S_un_w AS S_un_w_type
    s_addr AS DWORD
    s AS STRING * 4
END UNION

#IF %DEF(%USEMACROS) '---------------------------------------------------------

    ' can be used for most tcp & ip code
    MACRO s_addrm = S_un.S_addr

    ' host on imp
    MACRO s_hostm = S_Un.S_un_b.s_b2

    ' network
    MACRO s_netm = S_Un.S_un_b.s_b1

    ' imp
    MACRO s_impm = S_un.S_un_w.s_w2

    ' imp #
    MACRO s_impnom = S_un.S_un_b.s_b4

    ' logical host
    MACRO s_lhm = S_un.S_un_b.s_b3

#ENDIF  ' #IF %DEF(%USEMACROS) ------------------------------------------------

' Definitions of bits in internet address integers.
' On subnets, the decomposition of addresses to host and net parts
' is done according to subnet mask, not the masks here.

#IF %DEF(%USEMACROS)
    MACRO IN_CLASSA (i) = ((i AND &H80000000???) = 0)
#ENDIF

%IN_CLASSA_NET    = &HFF000000???
%IN_CLASSA_NSHIFT = 24
%IN_CLASSA_HOST   = &H00FFFFFF
%IN_CLASSA_MAX    = 128

#IF %DEF(%USEMACROS)
    MACRO IN_CLASSB (i) = ((i AND &HC0000000???) = &H80000000???)
#ENDIF

%IN_CLASSB_NET    = &HFFFF0000???
%IN_CLASSB_NSHIFT = 16
%IN_CLASSB_HOST   = &H0000FFFF
%IN_CLASSB_MAX    = 65536

#IF %DEF(%USEMACROS)
    MACRO IN_CLASSC (i) = ((i AND &HE0000000???) = &HC0000000???)
#ELSE
    FUNCTION IN_CLASSC (BYVAL i AS DWORD) AS LONG
        FUNCTION = ((i AND &He0000000???) = &Hc0000000???)
    END FUNCTION
#ENDIF

%IN_CLASSC_NET    = &HFFFFFF00???
%IN_CLASSC_NSHIFT = 8
%IN_CLASSC_HOST   = &H000000FF

%INADDR_ANY       = &H00000000
%INADDR_LOOPBACK  = &H7F000001
%INADDR_BROADCAST = &HFFFFFFFF???
%INADDR_NONE      = &HFFFFFFFF???

' Socket address, internet style.
TYPE sockaddr_in
    sin_family AS INTEGER
    sin_port AS WORD
    sin_addr AS in_addr
    sin_zero AS STRING * 8
END TYPE

%WSADESCRIPTION_LEN = 256
%WSASYS_STATUS_LEN  = 128

TYPE WSAdata
    wVersion       AS WORD
    wHighVersion   AS WORD
    szDescription  AS ASCIIZ * (%WSADESCRIPTION_LEN + 1)
    xFiller1       AS STRING * 3
    szSystemStatus AS ASCIIZ * (%WSASYS_STATUS_LEN + 1)
    xFiller2       AS STRING * 3
    iMaxSockets    AS WORD
    iMaxUdpDg      AS WORD
    lpVendorInfo   AS ASCIIZ PTR
END TYPE

' Options for use with [gs]etsockopt at the IP level.

%IP_OPTIONS         = 1           ' set/get IP per-packet options
%IP_MULTICAST_IF    = 2           ' set/get IP multicast interface
%IP_MULTICAST_TTL   = 3           ' set/get IP multicast timetolive
%IP_MULTICAST_LOOP  = 4           ' set/get IP multicast loopback
%IP_ADD_MEMBERSHIP  = 5           ' add  an IP group membership
%IP_DROP_MEMBERSHIP = 6           ' drop an IP group membership
%IP_TTL             = 7           ' set/get IP Time To Live
%IP_TOS             = 8           ' set/get IP Type Of Service
%IP_DONTFRAGMENT    = 9           ' set/get IP Don't Fragment flag

%IP_DEFAULT_MULTICAST_TTL  = 1    ' normally limit m'casts to 1 hop
%IP_DEFAULT_MULTICAST_LOOP = 1    ' normally hear sends if a member
%IP_MAX_MEMBERSHIPS        = 20   ' per socket; must fit in one mbuf

' Argument structure for IP_ADD_MEMBERSHIP and IP_DROP_MEMBERSHIP.
TYPE ip_mreq
    imr_multiaddr AS in_addr  ' IP multicast address of group
    imr_interface AS in_addr  ' local IP address of interface
END TYPE

' Definitions related to sockets: types, address families, options,
' taken from the BSD file sys/socket.h.

' This is used instead of -1, since the
' SOCKET type is unsigned.

%INVALID_SOCKET = &HFFFFFFFF???
%SOCKET_ERROR   = -1

' Types

%SOCK_STREAM    = 1                ' stream socket
%SOCK_DGRAM     = 2                ' datagram socket
%SOCK_RAW       = 3                ' raw-protocol interface
%SOCK_RDM       = 4                ' reliably-delivered message
%SOCK_SEQPACKET = 5                ' sequenced packet stream

' Option flags per-socket.

%SO_DEBUG       = &H00001          ' turn on debugging info recording
%SO_ACCEPTCONN  = &H00002          ' socket has had listen()
%SO_REUSEADDR   = &H00004          ' allow local address reuse
%SO_KEEPALIVE   = &H00008          ' keep connections alive
%SO_DONTROUTE   = &H00010          ' just use interface addresses
%SO_BROADCAST   = &H00020          ' permit sending of broadcast msgs
%SO_USELOOPBACK = &H00040          ' bypass hardware when possible
%SO_LINGER      = &H00080          ' linger on close if data present
%SO_OOBINLINE   = &H00100          ' leave received OOB data in line

%SO_DONTLINGER  = &H0FF7F          ' CDWD(NOT %SO_LINGER)

' Additional options.

%SO_SNDBUF      = &H01001          ' send buffer size
%SO_RCVBUF      = &H01002          ' receive buffer size
%SO_SNDLOWAT    = &H01003          ' send low-water mark
%SO_RCVLOWAT    = &H01004          ' receive low-water mark
%SO_SNDTIMEO    = &H01005          ' send timeout
%SO_RCVTIMEO    = &H01006          ' receive timeout
%SO_ERROR       = &H01007          ' get error status and clear
%SO_TYPE        = &H01008          ' get socket type

' Options for connect and disconnect data and options.  Used only by
' non-TCP/IP transports such as DECNet, OSI TP4, etc.

%SO_CONNDATA    = &H07000
%SO_CONNOPT     = &H07001
%SO_DISCDATA    = &H07002
%SO_DISCOPT     = &H07003
%SO_CONNDATALEN = &H07004
%SO_CONNOPTLEN  = &H07005
%SO_DISCDATALEN = &H07006
%SO_DISCOPTLEN  = &H07007

' Option for opening sockets for synchronous access.

%SO_OPENTYPE    = &H07008

%SO_SYNCHRONOUS_ALERT    = &H10
%SO_SYNCHRONOUS_NONALERT = &H20


' Other NT-specific options.

%SO_MAXDG        = &H07009
%SO_MAXPATHDG    = &H0700A
%SO_UPDATE_ACCEPT_CONTEXT = &H0700B
%SO_CONNECT_TIME = &H0700C


' TCP options.

%TCP_NODELAY   = &H00001
%TCP_BSDURGENT = &H07000

' Address families.

%AF_UNSPEC    = 0           ' unspecified
%AF_UNIX      = 1           ' local to host (pipes, portals)
%AF_INET      = 2           ' internetwork: UDP, TCP, etc.
%AF_IMPLINK   = 3           ' arpanet imp addresses
%AF_PUP       = 4           ' pup protocols: e.g. BSP
%AF_CHAOS     = 5           ' mit CHAOS protocols
%AF_NS        = 6           ' XEROX NS protocols
%AF_IPX       = 6           ' IPX and SPX
%AF_ISO       = 7           ' ISO protocols
%AF_OSI       = %AF_ISO     ' OSI is ISO
%AF_ECMA      = 8           ' european computer manufacturers
%AF_DATAKIT   = 9           ' datakit protocols
%AF_CCITT     = 10          ' CCITT protocols, X.25 etc
%AF_SNA       = 11          ' IBM SNA
%AF_DECnet    = 12          ' DECnet
%AF_DLI       = 13          ' Direct data link interface
%AF_LAT       = 14          ' LAT
%AF_HYLINK    = 15          ' NSC Hyperchannel
%AF_APPLETALK = 16          ' AppleTalk
%AF_NETBIOS   = 17          ' NetBios-style addresses
%AF_VOICEVIEW = 18          ' VoiceView
%AF_FIREFOX   = 19          ' FireFox
%AF_UNKNOWN1  = 20          ' Somebody is using this!
%AF_BAN       = 21          ' Banyan

%AF_MAX       = 22

' Structure used by kernel to store most addresses.

TYPE sockaddr
    sa_family AS WORD         ' address family
    sa_data   AS STRING * 14  ' up to 14 bytes of direct address
END TYPE

' Structure used by kernel to pass protocol
' information in raw sockets.

TYPE sockproto
    sp_family   AS WORD  ' address family
    sp_protocol AS WORD  ' protocol
END TYPE

' Protocol families, same as address families for now.

%PF_UNSPEC    = %AF_UNSPEC
%PF_UNIX      = %AF_UNIX
%PF_INET      = %AF_INET
%PF_IMPLINK   = %AF_IMPLINK
%PF_PUP       = %AF_PUP
%PF_CHAOS     = %AF_CHAOS
%PF_NS        = %AF_NS
%PF_IPX       = %AF_IPX
%PF_ISO       = %AF_ISO
%PF_OSI       = %AF_OSI
%PF_ECMA      = %AF_ECMA
%PF_DATAKIT   = %AF_DATAKIT
%PF_CCITT     = %AF_CCITT
%PF_SNA       = %AF_SNA
%PF_DECnet    = %AF_DECnet
%PF_DLI       = %AF_DLI
%PF_LAT       = %AF_LAT
%PF_HYLINK    = %AF_HYLINK
%PF_APPLETALK = %AF_APPLETALK
%PF_VOICEVIEW = %AF_VOICEVIEW
%PF_FIREFOX   = %AF_FIREFOX
%PF_UNKNOWN1  = %AF_UNKNOWN1
%PF_BAN       = %AF_BAN

%PF_MAX       = %AF_MAX

' Structure used for manipulating linger option.

TYPE linger
    l_onoff  AS WORD   ' option on/off
    l_linger AS WORD   ' linger time
END TYPE

' Level number for (get/set)sockopt() to apply to socket itself.

%SOL_SOCKET = &H0FFFF   ' options for socket level

' Maximum queue length specifiable by listen.

%SOMAXCONN = 5

%MSG_OOB       = &H01    ' process out-of-band data
%MSG_PEEK      = &H02    ' peek at incoming message
%MSG_DONTROUTE = &H04    ' send without using routing tables

%MSG_MAXIOVLEN = 16

%MSG_PARTIAL   = &H08000 ' partial send or recv for message xport


' Define constant based on rfc883, used by gethostbyxxxx() calls.

%MAXGETHOSTSTRUCT = 1024

' Define flags to be used with the WSAAsyncSelect() call.

%FD_READ    = &H001
%FD_WRITE   = &H002
%FD_OOB     = &H004
%FD_ACCEPT  = &H008
%FD_CONNECT = &H010
%FD_CLOSE   = &H020

#IF NOT %DEF(%WSABASEERR) '----------------------------------------------------

' All Windows Sockets error constants are biased by WSABASEERR from
' the "normal"

%WSABASEERR = 10000

' Windows Sockets definitions of regular Microsoft C error constants

%WSAEINTR           = %WSABASEERR + 4
%WSAEBADF           = %WSABASEERR + 9
%WSAEACCES          = %WSABASEERR + 13
%WSAEFAULT          = %WSABASEERR + 14
%WSAEINVAL          = %WSABASEERR + 22
%WSAEMFILE          = %WSABASEERR + 24

' Windows Sockets definitions of regular Berkeley error constants

%WSAEWOULDBLOCK     = %WSABASEERR + 35
%WSAEINPROGRESS     = %WSABASEERR + 36
%WSAEALREADY        = %WSABASEERR + 37
%WSAENOTSOCK        = %WSABASEERR + 38
%WSAEDESTADDRREQ    = %WSABASEERR + 39
%WSAEMSGSIZE        = %WSABASEERR + 40
%WSAEPROTOTYPE      = %WSABASEERR + 41
%WSAENOPROTOOPT     = %WSABASEERR + 42
%WSAEPROTONOSUPPORT = %WSABASEERR + 43
%WSAESOCKTNOSUPPORT = %WSABASEERR + 44
%WSAEOPNOTSUPP      = %WSABASEERR + 45
%WSAEPFNOSUPPORT    = %WSABASEERR + 46
%WSAEAFNOSUPPORT    = %WSABASEERR + 47
%WSAEADDRINUSE      = %WSABASEERR + 48
%WSAEADDRNOTAVAIL   = %WSABASEERR + 49
%WSAENETDOWN        = %WSABASEERR + 50
%WSAENETUNREACH     = %WSABASEERR + 51
%WSAENETRESET       = %WSABASEERR + 52
%WSAECONNABORTED    = %WSABASEERR + 53
%WSAECONNRESET      = %WSABASEERR + 54
%WSAENOBUFS         = %WSABASEERR + 55
%WSAEISCONN         = %WSABASEERR + 56
%WSAENOTCONN        = %WSABASEERR + 57
%WSAESHUTDOWN       = %WSABASEERR + 58
%WSAETOOMANYREFS    = %WSABASEERR + 59
%WSAETIMEDOUT       = %WSABASEERR + 60
%WSAECONNREFUSED    = %WSABASEERR + 61
%WSAELOOP           = %WSABASEERR + 62
%WSAENAMETOOLONG    = %WSABASEERR + 63
%WSAEHOSTDOWN       = %WSABASEERR + 64
%WSAEHOSTUNREACH    = %WSABASEERR + 65
%WSAENOTEMPTY       = %WSABASEERR + 66
%WSAEPROCLIM        = %WSABASEERR + 67
%WSAEUSERS          = %WSABASEERR + 68
%WSAEDQUOT          = %WSABASEERR + 69
%WSAESTALE          = %WSABASEERR + 70
%WSAEREMOTE         = %WSABASEERR + 71

%WSAEDISCON         = %WSABASEERR + 101

' Extended Windows Sockets error constant definitions

%WSASYSNOTREADY     = %WSABASEERR+91
%WSAVERNOTSUPPORTED = %WSABASEERR+92
%WSANOTINITIALISED  = %WSABASEERR+93

' Error return codes from gethostbyname() and gethostbyaddr()
' (when using the resolver). Note that these errors are
' retrieved via WSAGetLastError() and must therefore follow
' the rules for avoiding clashes with error numbers from
' specific implementations or language run-time systems.
' For this reason the codes are based at %WSABASEERR + 1001.
' Note also that [WSA]NO_ADDRESS is defined only for
' compatibility purposes.

 ' Authoritative Answer: Host not found
%WSAHOST_NOT_FOUND =      %WSABASEERR + 1001
%HOST_NOT_FOUND =         %WSAHOST_NOT_FOUND

 ' Non-Authoritative: Host not found, or SERVERFAIL
%WSATRY_AGAIN =           %WSABASEERR + 1002
%TRY_AGAIN =              %WSATRY_AGAIN

 ' Non recoverable errors, FORMERR, REFUSED, NOTIMP
%WSANO_RECOVERY =         %WSABASEERR + 1003
%NO_RECOVERY =            %WSANO_RECOVERY

 ' Valid name, no data record of requested type
%WSANO_DATA =             %WSABASEERR + 1004
%NO_DATA =                %WSANO_DATA

#ENDIF  ' #IF %DEF(%WSABASEERR) -----------------------------------------------

#IF %DEF(%USEMACROS)
    h_errno = WSAGetLastError()
#ENDIF

%HOST_NOT_FOUND = %WSAHOST_NOT_FOUND
%TRY_AGAIN      = %WSATRY_AGAIN
%NO_RECOVERY    = %WSANO_RECOVERY
%NO_DATA        = %WSANO_DATA
' no address, look for MX record
%WSANO_ADDRESS  = %WSANO_DATA
%NO_ADDRESS     = %WSANO_ADDRESS


' Windows Sockets errors redefined as regular Berkeley error constants

%EWOULDBLOCK     = %WSAEWOULDBLOCK
%EINPROGRESS     = %WSAEINPROGRESS
%EALREADY        = %WSAEALREADY
%ENOTSOCK        = %WSAENOTSOCK
%EDESTADDRREQ    = %WSAEDESTADDRREQ
%EMSGSIZE        = %WSAEMSGSIZE
%EPROTOTYPE      = %WSAEPROTOTYPE
%ENOPROTOOPT     = %WSAENOPROTOOPT
%EPROTONOSUPPORT = %WSAEPROTONOSUPPORT
%ESOCKTNOSUPPORT = %WSAESOCKTNOSUPPORT
%EOPNOTSUPP      = %WSAEOPNOTSUPP
%EPFNOSUPPORT    = %WSAEPFNOSUPPORT
%EAFNOSUPPORT    = %WSAEAFNOSUPPORT
%EADDRINUSE      = %WSAEADDRINUSE
%EADDRNOTAVAIL   = %WSAEADDRNOTAVAIL
%ENETDOWN        = %WSAENETDOWN
%ENETUNREACH     = %WSAENETUNREACH
%ENETRESET       = %WSAENETRESET
%ECONNABORTED    = %WSAECONNABORTED
%ECONNRESET      = %WSAECONNRESET
%ENOBUFS         = %WSAENOBUFS
%EISCONN         = %WSAEISCONN
%ENOTCONN        = %WSAENOTCONN
%ESHUTDOWN       = %WSAESHUTDOWN
%ETOOMANYREFS    = %WSAETOOMANYREFS
%ETIMEDOUT       = %WSAETIMEDOUT
%ECONNREFUSED    = %WSAECONNREFUSED
%ELOOP           = %WSAELOOP
%ENAMETOOLONG    = %WSAENAMETOOLONG
%EHOSTDOWN       = %WSAEHOSTDOWN
%EHOSTUNREACH    = %WSAEHOSTUNREACH
%ENOTEMPTY       = %WSAENOTEMPTY
%EPROCLIM        = %WSAEPROCLIM
%EUSERS          = %WSAEUSERS
%EDQUOT          = %WSAEDQUOT
%ESTALE          = %WSAESTALE
%EREMOTE         = %WSAEREMOTE

DECLARE FUNCTION aaccept LIB "wsock32.dll" ALIAS "accept" (BYVAL s AS DWORD, address AS sockaddr_in, addrlen AS LONG) AS DWORD
DECLARE FUNCTION bind LIB "wsock32.dll" ALIAS "bind" (BYVAL s AS DWORD, address AS sockaddr_in, BYVAL namelen AS LONG) AS LONG
DECLARE FUNCTION closesocket LIB "wsock32.dll" ALIAS "closesocket" (BYVAL s AS DWORD) AS LONG
DECLARE FUNCTION cconnect LIB "wsock32.dll" ALIAS "connect" (BYVAL s AS DWORD, address AS sockaddr_in, BYVAL namelen AS LONG) AS LONG
DECLARE FUNCTION ioctlsocket LIB "wsock32.dll" ALIAS "ioctlsocket" (BYVAL s AS DWORD, BYVAL cmd AS LONG, argp AS DWORD) AS LONG
DECLARE FUNCTION getpeername LIB "wsock32.dll" ALIAS "getpeername" (BYVAL s AS DWORD, sname AS sockaddr_in, namelen AS LONG) AS LONG
DECLARE FUNCTION getsockname LIB "wsock32.dll" ALIAS "getsockname" (BYVAL s AS DWORD, sname AS sockaddr_in, namelen AS LONG) AS LONG
DECLARE FUNCTION getsockopt LIB "wsock32.dll" ALIAS "getsockopt" (BYVAL s AS DWORD, BYVAL level AS LONG, BYVAL optname AS LONG, optval AS ANY, optlen AS LONG) AS LONG
DECLARE FUNCTION htonl LIB "wsock32.dll" ALIAS "htonl" (BYVAL hostlong AS DWORD) AS DWORD
DECLARE FUNCTION htons LIB "wsock32.dll" ALIAS "htons" (BYVAL hostshort AS WORD) AS WORD
DECLARE FUNCTION inet_addr LIB "wsock32.dll" ALIAS "inet_addr" (cp AS ASCIIZ) AS DWORD
DECLARE FUNCTION inet_ntoa LIB "wsock32.dll" ALIAS "inet_ntoa" (byval inn AS DWORD) AS DWORD  ' returns ASCIIZ PTR
DECLARE FUNCTION listen LIB "wsock32.dll" ALIAS "listen" (BYVAL s AS DWORD, BYVAL backlog AS LONG) AS LONG
DECLARE FUNCTION ntohl LIB "wsock32.dll" ALIAS "ntohl" (BYVAL netlong AS DWORD) AS DWORD
DECLARE FUNCTION ntohs LIB "wsock32.dll" ALIAS "ntohs" (BYVAL netshort AS WORD) AS WORD
DECLARE FUNCTION rrecv LIB "wsock32.dll" ALIAS "recv" (BYVAL s AS DWORD, buf AS Any, BYVAL buflen AS LONG, BYVAL flags AS LONG) AS LONG
DECLARE FUNCTION recvfrom LIB "wsock32.dll" ALIAS "recvfrom" (BYVAL s AS LONG, buf AS ANY, BYVAL buflen AS LONG, BYVAL flags AS LONG, Sockstruct AS sockaddr_in, strulen AS LONG) AS LONG
DECLARE FUNCTION sselect LIB "wsock32.dll" ALIAS "select" (BYVAL nfds AS LONG, readfds AS fd_setstruc, writefds AS fd_setstruc, exceptfds AS fd_setstruc, tymeout AS timeval) AS LONG
DECLARE FUNCTION ssend LIB "wsock32.dll" ALIAS "send" (BYVAL s AS DWORD, buf AS Any, BYVAL buflen AS LONG, BYVAL flags AS LONG) AS LONG
DECLARE FUNCTION sendto LIB "wsock32.dll" ALIAS "sendto" (BYVAL s AS DWORD, buf AS ANY, BYVAL buflen AS LONG, BYVAL flags AS LONG, to_addr AS sockaddr_in, BYVAL tolen AS LONG) AS LONG
DECLARE FUNCTION setsockopt LIB "wsock32.dll" ALIAS "setsockopt" (BYVAL s AS DWORD, BYVAL level AS LONG, BYVAL optname AS LONG, optval AS Any, BYVAL optlen AS LONG) AS LONG
DECLARE FUNCTION shutdown LIB "wsock32.dll" ALIAS "shutdown" (BYVAL s AS DWORD, BYVAL how AS LONG) AS LONG
DECLARE FUNCTION socket LIB "wsock32.dll" ALIAS "socket" (BYVAL af AS DWORD, BYVAL s_type AS LONG, BYVAL protocol AS LONG) AS DWORD

' Database function prototypes

DECLARE FUNCTION gethostbyaddr LIB "wsock32.dll"  ALIAS "gethostbyaddr" (address AS LONG, BYVAL addr_len AS LONG, BYVAL addr_type AS LONG) AS LONG
DECLARE FUNCTION gethostbyname LIB "wsock32.dll" ALIAS "gethostbyname" (ost_name AS ASCIIZ) AS LONG
DECLARE FUNCTION gethostname LIB "wsock32.dll" ALIAS "gethostname" (host_name AS ASCIIZ, BYVAL namelen AS LONG) AS LONG
DECLARE FUNCTION getservbyport LIB "wsock32.dll" ALIAS "getservbyport" (BYVAL lPort AS LONG, proto AS ASCIIZ) AS DWORD  ' returns servent ptr
DECLARE FUNCTION getservbyname LIB "wsock32.dll" ALIAS "getservbyname" (serv_name AS ASCIIZ, proto AS ASCIIZ) AS DWORD  ' returns servent ptr
DECLARE FUNCTION getprotobynumber LIB "wsock32.dll" ALIAS "getprotobynumber" (BYVAL proto AS LONG) AS DWORD   ' returns protoent ptr
DECLARE FUNCTION getprotobyname LIB "wsock32.dll" ALIAS "getprotobyname" (proto_name AS ASCIIZ) AS DWORD  ' returns protoent ptr

' Microsoft Windows Extension function prototypes

DECLARE FUNCTION WSAStartup LIB "wsock32.dll" ALIAS "WSAStartup" (BYVAL wVR AS WORD, lpWSAD AS WSAData) AS LONG
DECLARE FUNCTION WSACleanup LIB "wsock32.dll" ALIAS "WSACleanup" () AS LONG
DECLARE SUB      WSASetLastError LIB "wsock32.dll" ALIAS "WSASetLastError" (BYVAL iError AS LONG)
DECLARE FUNCTION WSAGetLastError LIB "wsock32.dll" ALIAS "WSAGetLastError" () AS LONG
DECLARE FUNCTION WSAIsBlocking LIB "wsock32.dll" ALIAS "WSAIsBlocking" () AS LONG
DECLARE FUNCTION WSAUnhookBlockingHook LIB "wsock32.dll" ALIAS "WSAUnhookBlockingHook" () AS LONG
DECLARE FUNCTION WSASetBlockingHook LIB "wsock32.dll" ALIAS "WSASetBlockingHook" (BYVAL lpBlockFunc AS DWORD) AS DWORD
DECLARE FUNCTION WSACancelBlockingCall LIB "wsock32.dll" ALIAS "WSACancelBlockingCall" () AS LONG
DECLARE FUNCTION WSAAsyncGetServByName LIB "wsock32.dll" ALIAS "WSAAsyncGetServByName" (BYVAL hWnd AS DWORD, BYVAL wMsg AS DWORD, serv_name AS ASCIIZ, proto AS ASCIIZ, buf AS ASCIIZ, BYVAL buflen AS LONG) AS DWORD
DECLARE FUNCTION WSAAsyncGetServByPort LIB "wsock32.dll" ALIAS "WSAAsyncGetServByPort" (BYVAL hWnd AS DWORD, BYVAL wMsg AS DWORD, BYVAL lPort AS LONG, proto AS ASCIIZ, buf AS ASCIIZ, BYVAL buflen AS LONG) AS DWORD
DECLARE FUNCTION WSAAsyncGetProtoByName LIB "wsock32.dll" ALIAS "WSAAsyncGetProtoByName" (BYVAL hWnd AS DWORD, BYVAL wMsg AS DWORD, proto_name AS ASCIIZ, buf AS ASCIIZ, BYVAL buflen AS LONG) AS DWORD
DECLARE FUNCTION WSAAsyncGetProtoByNumber LIB "wsock32.dll" ALIAS "WSAAsyncGetProtoByNumber" (BYVAL hWnd AS DWORD, BYVAL wMsg AS DWORD, BYVAL number AS LONG, buf AS ASCIIZ, BYVAL buflen AS LONG) AS DWORD
DECLARE FUNCTION WSAAsyncGetHostByName LIB "wsock32.dll" ALIAS "WSAAsyncGetHostByName" (BYVAL hWnd AS DWORD, BYVAL wMsg AS DWORD, host_name AS ASCIIZ, buf AS ASCIIZ, BYVAL buflen AS LONG) AS DWORD
DECLARE FUNCTION WSAAsyncGetHostByAddr LIB "wsock32.dll" ALIAS "WSAAsyncGetHostByAddr" (BYVAL hWnd AS DWORD, BYVAL wMsg AS DWORD, address AS ASCIIZ, BYVAL addr_len AS LONG, BYVAL addr_type AS LONG, buf AS ASCIIZ, BYVAL buflen AS LONG) AS DWORD
DECLARE FUNCTION WSACancelAsyncRequest LIB "wsock32.dll" ALIAS "WSACancelAsyncRequest" (BYVAL hAsyncTaskHandle AS DWORD) AS LONG
DECLARE FUNCTION WSAAsyncSelect LIB "wsock32.dll" ALIAS "WSAAsyncSelect" (BYVAL s AS DWORD, BYVAL hWnd AS DWORD, BYVAL wMsg AS DWORD, BYVAL lEvent AS LONG) AS LONG
DECLARE FUNCTION WSARecvEx LIB "wsock32.dll" ALIAS "WSARecvEx" (BYVAL s AS DWORD, buf AS Any, BYVAL buflen AS LONG, BYVAL flags AS LONG) AS LONG

TYPE TRANSMIT_FILE_BUFFERS
    Head       AS DWORD
    HeadLength AS DWORD
    Tail       AS DWORD
    TailLength AS DWORD
END TYPE

%TF_DISCONNECT   = &H01
%TF_REUSE_SOCKET = &H02
%TF_WRITE_BEHIND = &H04

DECLARE FUNCTION TransmitFile LIB "wsock32.dll" ALIAS "TransmitFile" (BYVAL hSocket AS DWORD, BYVAL hFile AS DWORD, BYVAL nNumberOfBytesToWrite AS DWORD, BYVAL nNumberOfBytesToSend AS DWORD, lpOverlapped AS OVERLAPPED, _
    lpTransmitBuffers AS TRANSMIT_FILE_BUFFERS, BYVAL dwReserved AS DWORD) AS LONG

DECLARE FUNCTION AcceptEx LIB "wsock32.dll" ALIAS "AcceptEx" (BYVAL sListenSocket AS DWORD, BYVAL sAcceptSocket AS DWORD, lpOutputBuffer AS ANY, BYVAL dwReceiveDataLength AS DWORD, BYVAL dwLocalAddressLength AS DWORD, _
    BYVAL dwRemoteAddressLength AS DWORD, lpdwBytesReceived AS DWORD, lpOverlapped AS OVERLAPPED) AS LONG

DECLARE SUB      GetAcceptExSockaddrs LIB "wsock32.dll" ALIAS "GetAcceptExSockaddrs" (lpOutputBuffer AS ANY, BYVAL dwReceiveDataLength AS DWORD, BYVAL dwLocalAddressLength AS DWORD, BYVAL dwRemoteAddressLength AS DWORD, _
    LocalSockaddr AS DWORD, LocalSockaddrLength AS LONG, RemoteSockaddr AS DWORD, RemoveSockaddrLength AS LONG)



#IF %DEF(%USEMACROS) '---------------------------------------------------------

    ' Windows message parameter composition and decomposition
    ' macros.
    '
    ' WSAMAKEASYNCREPLY is intended for use by the Windows Sockets implementation
    ' when constructing the response to a WSAAsyncGetXByY() routine.

    MACRO WSAMAKEASYNCREPLY (buflen, errorno) = MAKLNG(buflen, errorno)

    ' WSAMAKESELECTREPLY is intended for use by the Windows Sockets implementation
    ' when constructing the response to WSAAsyncSelect().

    MACRO WSAMAKESELECTREPLY(event, errorno) = MAKLNG(event, errorno)

    ' WSAGETASYNCBUFLEN is intended for use by the Windows Sockets application
    ' to extract the buffer length from the lParam in the response
    ' to a WSAGetXByY().

    MACRO WSAGETASYNCBUFLEN(lParam) = LOWRD(lParam)

    ' WSAGETASYNCERROR is intended for use by the Windows Sockets application
    ' to extract the error code from the lParam in the response
    ' to a WSAGetXByY().

    MACRO WSAGETASYNCERROR(lParam) = HIWRD(lParam)

    ' WSAGETSELECTEVENT is intended for use by the Windows Sockets application
    ' to extract the event code from the lParam in the response
    ' to a WSAAsyncSelect().

    MACRO WSAGETSELECTEVENT(lParam) = LOWRD(lParam)

    ' WSAGETSELECTERROR is intended for use by the Windows Sockets application
    ' to extract the error code from the lParam in the response
    ' to a WSAAsyncSelect().

    MACRO WSAGETSELECTERROR(lParam) = HIWRD(lParam)

    ' Return a pointer to all of the host IP addresses

    MACRO FUNCTION GetHostAddr

        MACROTEMP hostent, hostname, liststart

        LOCAL hostent AS hostentStru PTR
        LOCAL hostname AS ASCIIZ * 32
        LOCAL liststart AS DWORD

        IF GetHostName(hostname, SIZEOF(hostname)) THEN
            IF LEN(hostname) THEN
                hostent = GetHostByName(hostname)
                IF hostent THEN
                    liststart = @hostent.h_list
                END IF
            END IF
        END IF

    END MACRO = liststart

#ELSE  ' #IF %DEF(%USEMACROS) ----------------------------------------

    ' Return a pointer to all of the host IP addresses
    FUNCTION GetHostAddr () AS DWORD

        LOCAL hostent  AS hostentStru PTR
        LOCAL hostname AS ASCIIZ * 32

        IF GetHostName(hostname, SIZEOF(hostname)) THEN
            IF LEN(hostname) THEN
                hostent = GetHostByName(hostname)
                IF hostent THEN
                    FUNCTION = @hostent.h_list
                END IF
            END IF
        END IF

    END FUNCTION

#ENDIF  ' #IF %DEF(%USEMACROS) ------------------------------------------------



' The ICMP functions shown below are documented in MSDN, but are not actually
' shown in the Microsoft SDK files. Use at your own risk!

DECLARE FUNCTION IcmpCloseHandle LIB "ICMP.DLL" ALIAS "IcmpCloseHandle" (BYVAL IcmpHandle AS DWORD) AS LONG
DECLARE FUNCTION IcmpCreateFile LIB "ICMP.DLL" ALIAS "IcmpCreateFile" () AS DWORD
DECLARE FUNCTION IcmpSendEcho LIB "ICMP.DLL" ALIAS "IcmpSendEcho" (BYVAL IcmpHandle AS DWORD, BYVAL DestinationAddress AS LONG, RequestData AS ASCIIZ, BYVAL RequestSize AS WORD, RequestOptions AS ANY, ReplyBuffer AS ANY, _
    BYVAL ReplySize AS DWORD, BYVAL Tymeout AS DWORD) AS DWORD



#ENDIF  ' #IF NOT %DEF(%WSOCK32_INC)
