'
'	cmdtwit.bas
'
'	command line twitter status updater
'
'	by don dickinson - www.greatwebdivide.com
'	hereby freeware
'
'	do with it as you wish. give credit where its do.
'
'	-don
'
#compile exe "c:\tools\cmdtwit.exe"
#dim all

#include "cmdtwit.inc"
#include "pb_file.bas"
#include "pb_ini.bas"
#include "pb_cmd.bas"
#include "pb_date.bas"
#include "pb_enc.bas"
#include "pb_url.bas"
#include "pb_mime2.bas"
#include "pb_tcpip.bas"

global g_debug as long
global g_verbose as long
global g_appPath as string
global g_iniFile as string

global g_webApp as string
global g_webServer as string

global g_twitAccount as string
global g_twitPassword as string
global g_twitStatus as string


$DEFAULT_WEBAPP = "/statuses/update.xml?status="
$DEFAULT_WEBSERVER = "www.twitter.com"

$INI_FILE = "cmdtwit.ini"

$ENC_PASSWORD = ")(*!@#KJLKJjjkjkeeea34E3edLK"

%ERROR_BAD_PARAMETERS = 1


'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  conOut
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
sub conOut( sMsg as string )

	stdout sMsg

end sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  verboseOut
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
sub verboseOut ( sMsg as string )

   if g_verbose then
      stdout sMsg
   end if

end sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  debugOut
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
sub debugOut ( sMsg as string )

   if g_debug then outputDebugString sMsg + $nul
   verboseOut sMsg

end sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  logOut
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
sub logOut ( sMsg as string )

   dim fileName as string

   fileName = extractFileName(getExeName())
   replace "." with "_" in fileName
   fileName = g_appPath + fileName + ".log." + left$(dbfDate(date$), 6)
   pbAppendLog fileName + $nul, sMsg + $nul

end sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'	displayUsage
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
sub displayUsage

	print "CMDTWIT.EXE"
	print "Freeware from Don Dickinson"
	print "www.greatwebdivide.com"
	print ""
	print "Updates twitter status from the command line"
	print ""
	print "parameters:"
	print "/account=<twitter account>"
	print "/password=<twitter password>"
	print "/status=<twitter status message>"
	print "/save"
	print "/verbose"
	print "/silent"
	print ""
	print "NOTES "
	print "      This software will save your account and password to the "
	print "      cmdtwit.ini file (encrypting the password) so that you don't "
	print "      have to specify them next time it is run. It will only do "
	print "      this if you specify the /save command parameter."
	print ""
	print "      If you do not specify the account name, the software will try"
	print "      to use the account it last used. This will only work if the"
	print "      password was saved."
	print ""
	print "EXAMPLE"
	print "cmdtwit.exe /account='my account' /password='this is my password' "
	print "      /status='Here comes my status' /save"
	print ""
	print "Since i saved the password the first time, i can now call it like"
	print "this from now on:"
	print "cmdtwit.exe /status='My new status'"
	print ""

end sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  initParams
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function initParams() as long

   g_appPath = extractPath(getExeName())
   g_debug = cmdNameExists("debug")
   g_verbose = cmdNameExists("verbose")

	g_iniFile = $INI_FILE
	g_webServer = trim$(getIni(g_iniFile, "general", "web-server", $DEFAULT_WEBSERVER))
	if g_webServer = "" then
		g_webSErver= $DEFAULT_WEBSERVER
	end if

	g_webApp = trim$(getIni(g_iniFile, "general", "web-app", $DEFAULT_WEBAPP))
	if g_webApp = "" then
		g_webApp = $DEFAULT_WEBAPP
	end if

	g_twitAccount = trim$(cmdValue("account"))
	if g_twitAccount = "" then
		g_twitAccount = trim$(getIni(g_iniFile, "general", "last-account", ""))
		if g_twitAccount = "" then
			conOut("ERROR - If you have never run the program before, " + _
					"then you need to specify the /account parameter")
			exit function
		end if
	end if

	'- see if the user passed the password on the command line
	g_twitPassword = trim$(cmdValue("password"))
	if g_twitPassword = "" then
		g_twitPassword = trim$(getIni(g_iniFile, g_twitAccount, "password", ""))
		if g_twitPassword = "" then
			conOut("ERROR - No password found saved for this account and no /password " + _
					"parameter specified")
			exit function
		end if
		g_twitPassword = crypt_string(decode_ascii(g_twitPassword), $ENC_PASSWORD)
	end if

	g_twitStatus = trim$(cmdValue("status"))
	if g_twitStatus = "" then
		conOut("ERROR - you must specify the /status parameter")
		exit function
	end if

   function = %true

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'	saveSettings
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
sub saveSettings

	if cmdNameExists("save") then
		writeIni g_iniFile, "general", "last-account", g_twitAccount
		writeIni g_iniFile, g_twitAccount, "password", _
				code_ascii(crypt_string(g_twitPassword, $ENC_PASSWORD))
	end if

end sub


'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  pbmain
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function pbmain

	dim sCmd as string
	dim sResult as string
	dim mimeString as string


   function = 1		'error

	if trim$(command$) = "" then
		displayUsage
		exit function
	end if

   cmdInit
   if initParams() = %false then
      function = %ERROR_BAD_PARAMETERS
      goto pbmain_done
   end if
	saveSettings

	'stdout "webApp: " + g_webApp
	'stdout "webServer: " + g_webServer
	'stdout "twitAccount: " + g_twitAccount
	'stdout "twitPassword: " + g_twitPassword
	'stdout "twitStatus: " + g_twitStatus
	mimeEncode g_twitAccount + ":" + g_twitPassword, mimeString

	sCmd = "POST " + g_webApp + urlEncode(g_twitStatus) + " HTTP/1.0" + $crlf + _
          "Authorization: Basic " + mimeString + $crlf + _
          "Accept: */*" + $crlf + _
          "Accept-Language: en-us" + $crlf + _
          "Host: " + g_webServer + $crlf + _
          $crlf

   if httpSendAndReceive(g_webServer, sCmd, sResult) = %false then
      conOut("ERROR - unable to talk with web server")
   elseif instr(sResult, "<error>") then
   	conOut("Twitter error: " + extract$(remain$(sResult, "<error>"), "</error>"))
   else
		'- all is ok
		function = 0
      conOut("Success")
	end if


pbmain_done:
   cmdFree

end function
