'
'	ff_pb_cmd.bas
'
'
'  DESCRIPTION
'     Parses command line parameters in the form
'        program.exe /P:param1 /Q:"parameter number 2"
'
'     Call cmdInit first
'     then use cmdValue(), cmdCount(), cmdNameN(), cmdValueN()
'     to access them.
'
'  AUTHOR
'     Don Dickinson
'     ddickinson@usinternet.com
'     www.basicguru.com/dickinson
'
'  LICENSE and DISCLAIMER
'     Hereby Public Domain
'     Use this code as you see fit. By using or compiling this code or derivative
'     thereof, you are consenting to the hold the author, Don Dickinson, harmless
'     for all effects or side-effects its use. This code works great for me,
'     but you are using it at your own risk.
'
'  DEPENDS ON
'     dd_link.bas    'linked list handling
'
Global g_pb_cmd_Params as Long

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  cmdInit2
'
'  initializes the command line parameters from a string not from command
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
sub cmdInit2 alias "cmdInit2" ( sParams as string ) export

   Dim i as Long
   Dim sCmd as String
   Dim newPart as String
   Dim cleanCmd as String
   Dim oneCmd as String
   Dim sName as String
   Dim sValue as String

	if g_pb_cmd_params then llFree g_pb_cmd_params

   sCmd = trim$(sParams) + chr$(0)
   g_pb_cmd_Params = llAdd(0, "list root", sCmd)

   '- Now break it up

   '- First get rid of the spaces inside of quotes
   cleanCmd = ""

   '- get rid of double-quotes
   replace $dq + $dq with chr$(2) in sCmd

   For i = 1 to parseCount(sCmd, $dq)
      newPart = Parse$(sCmd, $dq, i)
      if i mod 2 = 0 then
         replace " " with chr$(1) in newPart
         replace "/" with chr$(3) in newPart
      end if
      cleanCmd = cleanCmd + newPart
   Next i

   '- Now we have a clean command line that can be
   '  parsed on the slash (/)
   '
   For i = 1 to parseCount(cleanCmd, "/")
      oneCmd = parse$(cleanCmd, "/", i)
      sName = trim$(extract$(oneCmd, any ":="))
      sValue = remain$(oneCmd, any ":=")

      '- Put back the spaces
      replace chr$(1) with " " in sValue
		replace chr$(2) with $dq in sValue
		replace chr$(3) with "/" in sValue
      if sName <> "" then
         llAdd g_pb_cmd_Params, sName, sValue
      end if
   Next i
end sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  cmdInit
'
'  Parses the command line parameters and puts them in the linked list.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Sub cmdInit Alias "cmdInit" Export

   dim sCmd as string

   sCmd = command$

   cmdInit2 sCmd

End Sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  cmdCount
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function cmdCount Alias "cmdCount" Export as Long

   '- Don't include the root node
   Function = llCount(g_pb_cmd_Params) - 1

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  cmdName
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function cmdName Alias "cmdName" _
      ( ByVal iParamNumber as Long ) Export as String

   Dim pNode as Long

   pNode = llGetByNumber(g_pb_cmd_Params, iParamNumber + 1)
   if pNode = 0 then
      Function = ""
   else
      Function = trim$(llName(pNode))
   end if

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  cmdNameExists
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function cmdNameExists Alias "cmdNameExists" _
      ( sName as String ) Export as Long

   Dim bFound as Long
   Dim pNext as Long

   sName = trim$(lcase$(sName))
   bFound = %false
   pNext = g_pb_cmd_Params
   do until pNext = 0
      if trim$(lcase$(llName(pNext))) = sName then
         bFound = %true
         exit do
      end if
      pNext = llNext(pNext)
   loop

   Function = bFound

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  cmdValue
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Function cmdValue Alias "cmdValue" _
      ( sParamName as String) Export as String

   Function = llDataByName(g_pb_cmd_Params, sParamName)

End Function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  cmdFree
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Sub cmdFree Alias "cmdFree" export

   if g_pb_cmd_Params then llFree g_pb_cmd_Params

End Sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'	cmdReadFile
'
'	this can read a command line from a file in case its too long
'	to actually be used.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
sub cmdReadFile ( byval fileName as string ) export

	dim iFF as long
	dim iLen as long
	dim fileData as string

	if trim$(dir$(fileName)) <> "" then
		iFF = freeFile
		open fileName for binary as #iFF
		if err then
			exit sub
		end if
		iLen = lof(iFF)
		if iLen > 0 then
			fileData = space$(iLen)
			get #iFF,, fileData
			cmdInit2 fileData
		end if
		close #iFF
	end if

end sub
