'
'  ff_pb_file.bas
'
'	Public domain files from Don Dickinson
'	don@greatwebdivide.com
'
'  PB-32bit misc file routines
'
'  Function FileDate(ByVal sFileName as String, ByVal sFormat as String as String
'  Function FileSize(ByVal sFileName as String) as Long
'  Function FileExists(sFile As String) As Long
'  Function ExtractPath (zFullFile As String) As String
'  Function ExtractFileName(zFillFile as String) as String
'  Function GetExeName() As String
'  Function GetModuleName(ByVal sModule as String) as String
'  Function GetTempFile(ByVal sSeed, ByVal sExt as String) As String
'  Function GetFileDateTime(sFileName as String) as String
'  Function GetFileSize(sFileName as String) as String
'  Function GetWindowsDir() as String
'  Function GetSystemDir() as String
'  Function GetTemporaryDir() as String
'  Sub DeleteFileSpec(ByVal sFileSpec As String)
'  Sub EnsureDirectoryExists(ByVal sFileName As String)
'  Function pbWriteFile(filename,filedata) as Long
'  Function ShellDocument(filename)
'
'  DEPENDS UPON
'     win32api.inc
'

global g_pb_file_LastError as string

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  pbFileLastError
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function pbFileLastError alias "pbFileLastError" export as string

   function = g_pb_file_LastError

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  FileDate
'
'  Returns file date as a string formatted in the specified manner.
'  This is the date which the file was last written to.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function FileDate alias "FileDate" _
      ( byval sFileName as string, byval sFormat as string) export as string

   dim hFind as long
   dim ampm as string
   dim rData as WIN32_FIND_DATA
   dim rTime as SYSTEMTIME
	dim rTimeZone as TIME_ZONE_INFORMATION

   hFind = findfirstfile(sFileName + $nul, rData)
   if hFind = %INVALID_HANDLE_VALUE then
      function = "n/a"
   else
      FileTimeToSystemTime rData.ftLastWriteTime, rTime
		getTimeZoneInformation rTimeZone
		systemTimeToTzSpecificLocalTime rTimeZone, rTime, rTime

      sFormat = ucase$(sFormat)
      if instr(sFormat, "A/P") > 0 then
         if rTime.wHour > 12 then
            rTime.wHour = rTime.wHour - 12
            ampm = "p"
         elseif rTime.wHour = 12 then
            ampm = "p"
         else
            ampm = "a"
         end if

      elseif instr(sFormat, "AM/PM") > 0 then
         if rTime.wHour > 12 then
            rTime.wHour = rTime.wHour - 12
            ampm = "pm"
         elseif rTime.wHour = 12 then
            ampm = "pm"
         else
            ampm = "am"
         end if
      else
         ampm = ""
      end if

      if len(ampm) > 0 then
         replace "A/P" with ampm in sFormat
         replace "AM/PM" with ampm in sFormat
      end if

      replace "YYYY" with format$(rTime.wYear, "0000") in sFormat
      replace "YY" with right$(format$(rTime.wYear, "0000"), 2) in sFormat
      replace "MM" with format$(rTime.wMonth, "00") in sFormat
      replace "M" with format$(rTime.wMonth, "0") in sFormat
      replace "DD" with format$(rTime.wDay, "00") in sFormat
      replace "D" with format$(rTime.wDay) in sFormat
      replace "HH" with format$(rTime.wHour, "00") in sFormat
      replace "H" with format$(rTime.wHour) in sFormat
      replace "NN" with format$(rTime.wMinute, "00") in sFormat
      replace "N" with format$(rTime.wMinute) in sFormat
      replace "SS" with format$(rTime.wSecond, "00") in sFormat
      replace "S" with format$(rTime.wSecond) in sFormat
      function = sFormat
      findclose hFind
   end if

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  FileSize
'
'  Returns file size in bytes or -1 if there's an error
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function FileSize alias "FileSize" (byval sFileName as string) export as long

   dim hFind as long
   dim rData as WIN32_FIND_DATA

   hFind = findfirstfile(sFileName + $nul, rData)
   if hFind = %INVALID_HANDLE_VALUE then
      function = -1
   else
      function = rData.nFileSizeLow
      findclose hFind
   end if

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  quadFileSize
'
'  Returns file size in bytes or -1 if there's an error
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function quadFileSize alias "quadFileSize" (byval sFileName as string) export as quad

   dim hFind as long
   dim rData as WIN32_FIND_DATA

   hFind = findfirstfile(sFileName + $nul, rData)
   if hFind = %INVALID_HANDLE_VALUE then
      function = -1
   else
      function = rData.nFileSizeLow + rData.nFileSizeHigh * (2 ^ 32)
      findclose hFind
   end if

end function


'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  ExtractFileName
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function ExtractFileName alias "ExtractFileName" _
      ( byval sFileName as string ) export as string

   function = parse$(sFileName, any "\/:", -1)

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  FileExists
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function FileExists alias "FileExists" _
      ( byval sFile as string ) export as long

   if dir$(sFile) = "" then
      function = %false
   else
      function = %true
   end if

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  EnsureDirectoryExists
'
'  Creates all subdirectories if they don't already exist.
'  You can specify a full file name and or just the path.
'  If just the path is specified, it must end in a backslash
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
sub EnsureDirectoryExists alias "EnsureDirectoryExists" _
      ( byval sFileName as string ) export

   dim i as long
   dim sDrive as string

   if instr(sFileName, "\") < 1 then exit sub

   '- All errors are ignored
   on error resume next
   sDrive = ""
   for i = 1 to parsecount(sFileName, "\") - 1
      if sDrive = "" then
         sDrive = parse$(sFileName, "\", 1)
      else
         sDrive = sDrive + "\" + parse$(sFileName, "\", i)
      end if
      mkdir sDrive
   next i

   errclear
   on error goto 0

end sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  ExtractPath
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function ExtractPath alias "ExtractPath" _
      ( byval sFullFile as string ) export as string

   '- This routine strips the file
   '  path from the full name.
   '
   dim i as long
   dim iStartAccum as long
   dim iLen as long
   dim sFile as string
   dim sAccum as string

   sFile = ltrim$(rtrim$(sFullFile))
   iLen = len(sFile)
   sAccum = ""

   iStartAccum = %False
   if iLen > 0 then
      for i = iLen to 1 step -1
         if (mid$(sFile, i, 1) = "\") or _
            (mid$(sFile, i, 1) = ":") then
            iStartAccum = %True
         end if
         if iStartAccum = %True then
            sAccum = mid$(sFile, i, 1) + sAccum
         end if
      next i
   end if

   sAccum = trim$(sAccum)
   if sAccum = "" then
   	sAccum = ".\"
   elseif right$(sAccum, 1) <> "\" then
		sAccum = sAccum + "\"
   end if

   function = sAccum

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  GetModuleName
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function GetModuleName alias "GetModuleName" _
      ( byval sModule as string ) export as string

   dim zFileName as asciiz * 400
	dim temp as string

   getmodulefilename getmodulehandle(bycopy sModule + $nul), zFileName, 400
   temp = trim$(zFilename)
   if left$(temp,4) = "\\?\" then temp = mid$(temp, 5)
   function = temp

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  GetExeName
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function GetExeName alias "GetExeName" export as string

   dim zFileName as asciiz * 400
	dim temp as string

   getmodulefilename byval %null, zFileName, 400
	temp = trim$(zFileName)
	if left$(temp,4) = "\\?\" then temp = mid$(temp, 5)
   function = temp

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  GetAppPath
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function GetAppPath alias "GetAppPath" () export  as string

   dim sPath as string

   sPath = trim$(ExtractPath(GetExeName()))
   if right$(sPath, 1) <> "\" then sPath = sPath + "\"
   function = sPath

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  GetTempFile
'
'  Takes a seed value (up to 4 characters)
'  and an extension (e.g. ".ddc") and
'  returns a unique file name.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function GetTempFile alias "GetTempFile" _
      ( byval sSeed as string, byval sExt as string ) export as string

   dim iCount as long
   dim sName as string
   dim sPath as string
   dim sTest as string
   dim sFormat as string
   dim zPath as asciiz * 300
   dim zSeed as asciiz * 100

   '- Get Temp Path
   gettemppath sizeof(zPath), zPath

   sPath = trim$(zPath)
   if right$(sPath, 1) <> "\" then
      sPath$ = sPath + "\"
   end if

   if len(sSeed) > 7 then sSeed = left$(sSeed,7)
   sFormat = string$(8 - len(sSeed), "0")
   sName = ""
   for iCount = 1 to 9999
      sTest = sPath + sSeed + format$(iCount, sFormat) + sExt
      if FileExists(sTest) = %False then
         sName = sTest
         exit for
      end if
   next iCount

   function = sName

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  GetTemporaryDir
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function GetTemporaryDir alias "GetTemporaryDir" export as string

   dim zPath as asciiz * 500
   dim sPath as string

   gettemppath sizeof(zPath), zPath
   sPath = trim$(zPath)
   if right$(sPath, 1) <> "\" then sPath = sPath + "\"
   function = sPath

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  GetWindowsDir
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function GetWindowsDir alias "GetWindowsDir" export as string

   dim zPath as asciiz * 500
   dim sPath as string

   getwindowsdirectory zPath, sizeof(zPath)
   sPath = trim$(zPath)
   if right$(sPath, 1) <> "\" then sPath = sPath + "\"
   function = sPath

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  GetSystemDir
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function GetSystemDir alias "GetSystemDir" export as string

   dim zPath as asciiz * 500
   dim sPath as string

   getsystemdirectory zPath, sizeof(zPath)
   sPath = trim$(zPath)
   if right$(sPath, 1) <> "\" then sPath = sPath + "\"
   function = sPath

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  collectFileSpec
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function collectFileSpec alias "collectFileSpec" _
      ( byval sFileSpec as string, sFiles() as string ) export as long

   dim i as long
   dim iCount as long
   dim sF as string
   dim sPath as string

   on error resume next

   redim sFiles(0 to 0) as string
   iCount = 0

   sPath = trim$(extractPath(sFileSpec))
   sF = dir$(sFileSpec)
   do until sF = ""
      iCount = iCount + 1
      redim preserve sFiles(0 to iCount) as string
      sFiles(iCount) = sPath + sF
      sF = dir$
   loop

   function = iCount

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  DeleteFileSpec
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
sub DeleteFileSpec alias "DeleteFileSpec" _
      ( byval sFileSpec as string ) export

   dim i as long
   dim iCount as long
   dim sF as string
   dim sPath as string
   dim sFiles() as string

   on error resume next

   redim sFiles(0 to 0) as string
   iCount = 0

   sPath = trim$(ExtractPath(sFileSpec))

   sF = dir$(sFileSpec)
   do until sF = ""
      iCount = iCount + 1
      redim preserve sFiles(0 to iCount) as string
      sFiles(iCount) = sPath + sF
      sF = dir$
   loop
   for i = 1 to iCount
      errclear
      kill sFiles(i)
   next i

end sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  pbWriteFile
'     Writes a block of binary data to the file specified. If the
'     file exists, this fails.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function pbWriteFile alias "pbWriteFile" _
      ( byval sFileName as string, byval sFileData as string ) _
        export as long

   dim iFF as long

   if dir$(sFileName) <> "" then
      g_pb_file_LastError = "Output file exists."
      function = %false
      exit function
   end if

   iFF = freefile
   open sFileName for binary lock read write as #iFF
   if err then
      g_pb_file_LastError = "Unable to open output file '" + _
            sFileName + "' error=" + format$(err)
      function = %false
      exit function
   end if

   put #iFF,, sFileData

   close #iFF

   function = %true

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  pbzWriteFile
'
'  Writes a block of binary data to the file specified. If the
'  file exists, this fails.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function pbzWriteFile alias "pbzWriteFile" _
      ( zFileName as asciiz, zFileData as asciiz ) export as long

   function = pbWriteFile( (zFileName), (zFileData) )

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  pbReadFile
'     Reads a file into a string
'		returns %true if successful or %false if it fails
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function pbReadFile alias "pbReadFile" _
      ( byval sFileName as string, sFileData as string ) _
        export as long

   dim iFF as long

	sFileData = ""

   if dir$(sFileName) = "" then
      g_pb_file_LastError = "Input file not found."
      function = %false
      exit function
   end if

   iFF = freefile
   open sFileName for binary lock read write as #iFF
   if err then
      g_pb_file_LastError = "Unable to open input file '" + _
            sFileName + "' error=" + format$(err) + ":" + error$
      function = %false
      exit function
   end if

	if lof(iFF) > 0 then
		sFileData = space$(lof(iFF))
	   get #iFF,, sFileData
	end if

   close #iFF

   function = %true

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  ShellDocument
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
sub ShellDocument alias "ShellDocument" (byval sFile as string) export

  shellexecute 0, byval %null, sFile + $nul, byval %null, byval %null, _
      %SW_NORMAL

end sub


'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  ShellDocumentWithCommand
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
sub ShellDocumentWithCommand alias "ShellDocumentWithCommand" _
		(byval sFile as string, byval cmdLine as string) export

  shellexecute 0, byval %null, sFile + $nul, cmdLine + $nul, byval %null, _
      %SW_NORMAL

end sub

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  RunWaitAndClose
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function RunWaitAndClose alias "RunWaitAndClose" _
      ( byval sCmdLine as string, byval sCurDir as string, _
        byval conTitle as string, byval iAllocConsole as long, _
        byval iShow as long ) as long

   dim SI as STARTUPINFO
   dim PI as PROCESS_INFORMATION
   dim ex as dword

   %CREATE_PROCESS_ERROR = -9999

   ex = 1

   if iAllocConsole then allocconsole

   SI.cb          = sizeof(SI)
   SI.dwFlags     = %STARTF_USESTDHANDLES or %STARTF_USESHOWWINDOW
   SI.hStdInput   = getstdhandle( %STD_INPUT_HANDLE )
   SI.hStdOutput  = getstdhandle( %STD_OUTPUT_HANDLE )
   SI.hStdError   = getstdhandle( %STD_ERROR_HANDLE )
   SI.wShowWindow = iShow

'
'    SI.hStdOutput  := CreateFile('c:\file_o.txt',
'                            GENERIC_WRITE, 0,
'                            nil, CREATE_ALWAYS, 0, 0);
'
'   SI.hStdError   := CreateFile('c:\file_e.txt',
'                            GENERIC_WRITE, 0,
'                            nil, CREATE_ALWAYS, 0, 0);
   if createprocess( "", sCmdLine + $nul, byval 0, byval 0, %true, 0, _
         byval 0, sCurDir + $nul, SI, PI ) then
      if conTitle <> "" then setconsoletitle( conTitle + $nul)

      do until waitforsingleobject(pi.hProcess, 100) <> %WAIT_TIMEOUT
         if iAllocConsole then
            sleep 100
         else
            apiDoevents
         end if
      loop

      if getexitcodeprocess(pi.hProcess, ex) = 0 then
         ex = getlasterror()
      end if
   else
      ex = %CREATE_PROCESS_ERROR
   end if

   if iAllocConsole then
      freeconsole
      closehandle SI.hStdOutput
      closehandle SI.hStdError
   end if

   function = ex

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  dirExists
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function dirExists(byval sDir as string) as long

   dim iCode as long

   errclear
   iCode = getattr(sDir)
   if err then
      function = %false
   elseif (iCode and %SUBDIR) = %SUBDIR then
      function = %true
   else
      function = %false
   end if

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  pbAppendLog
'
'  safely appends data to a log file. the file is opened exclusively
'  by protecting the open with a mutex.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function pbAppendLog alias "pbAppendLog" _
      ( zLogFile as asciiz, zData as asciiz ) export as long

   dim iFF as long

   if mtxLock( trim$(zLogFile), 3000 ) then
      errclear
      iFF = freefile
      open zLogFile for append as #iFF
      if err then
         function = %false
      else
         'print #iFF, date$ + " " + time$ + "  " + trim$(zData)
         print #iFF, trim$(zData)
         close #iFF
         function = %true
      end if
      mtxUnlock trim$(zLogFile)
   else
      function = %false
   end if

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  RunWaitAndCloseEx
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function RunWaitAndCloseEx alias "RunWaitAndCloseEx" _
      ( byval sCmdLine as string, byval sCurDir as string, _
        byval conTitle as string, byval iAllocConsole as long, _
        byval iShow as long, sStdout as string ) as long

   dim SI as STARTUPINFO
   dim PI as PROCESS_INFORMATION
   dim SA as SECURITY_ATTRIBUTES
   dim ex as dword
	dim tempFile as asciiz * %MAX_PATH

	sStdOut = ""

   %CREATE_PROCESS_ERROR = -9999

   ex = 1

   if iAllocConsole then allocconsole

   SI.cb          = sizeof(SI)
   SI.dwFlags     = %STARTF_USESTDHANDLES or %STARTF_USESHOWWINDOW		'
   SI.hStdInput   = getstdhandle( %STD_INPUT_HANDLE )
   'SI.hStdOutput  = GetStdHandle( %STD_OUTPUT_HANDLE )
   SI.hStdError   = getstdhandle( %STD_ERROR_HANDLE )
   SI.wShowWindow = iShow

	'- we have to create a security attributes structure in order
	'	to specify that the handle can be inherited. if we try to
	'	pass it as byval %null, it won't work because the handles
	'	cannot be inheritted.
	'
	SA.nLength = sizeof(SA)
	SA.lpSecurityDescriptor = %null
	SA.bInheritHandle = %true
	tempFile = getTempFile("~pb", ".tmp")
   SI.hStdOutput = createfile(tempFile, _
			%GENERIC_WRITE, 0, _
         SA, %CREATE_ALWAYS, 0, 0)
   if createprocess( "", sCmdLine + $nul, byval 0, byval 0, %true, 0, _
         byval 0, sCurDir + $nul, SI, PI ) then
      if conTitle <> "" then setconsoletitle( conTitle + $nul)

      do until waitforsingleobject(pi.hProcess, 100) <> %WAIT_TIMEOUT
         if iAllocConsole then
            sleep 100
         else
            apiDoevents
         end if
      loop

      if getexitcodeprocess(pi.hProcess, ex) = 0 then
         ex = getlasterror()
      end if
   else
      ex = %CREATE_PROCESS_ERROR
   end if

   if iAllocConsole then
      freeconsole
      closehandle SI.hStdOutput
      closehandle SI.hStdError
   else
   	closehandle SI.hStdOutput
   end if

	'- read back the stdoutput of the shelled process
	'	and delete the temporary file.
	'
	if fileExists(tempFile) then
		pbReadFile tempFile, sStdout
		kill tempFile
	end if
   function = ex

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'	getFileVersion
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function getFileVersion( byval fileName as string) as string

  	dim iLen as long
  	dim iFoo as long
  	dim iMajor as long
  	dim iMinor as long
  	dim iBuild as long
  	dim iSub as long
  	dim sBuffer as string
	dim rInfo as VS_FIXEDFILEINFO ptr

	iLen = getfileversioninfosize(fileName + $nul, iFoo)
	if iLen < 1 then
		function = ""
		exit function
	end if

	sBuffer = string$(iLen, $nul)
	if getfileversioninfo(fileName + $nul, 0, iLen, byval strptr(sBuffer)) = 0 then
		function = ""
		exit function
	end if

	verqueryvalue byval strptr(sBuffer), "\", rInfo, sizeof(rInfo)
  	iMajor = @rInfo.dwProductVersionMs \ &h10000
  	iMinor = @rInfo.dwProductVersionMs mod &h10000
  	iBuild = @rInfo.dwProductVersionLS mod &h10000
	iSub = @rInfo.dwProductVersionLS \  &h10000

	function = format$(iMajor) + "." + _
			format$(iMinor) + "." +_
			format$(iBuild) + "." + _
			format$(iSub)

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'	getLoggedInUser
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function getLoggedInUser() as string

   dim dwSize as dword
   dim zUser as asciiz * %MAX_PATH

   dwSize = %MAX_PATH
   if getusername(zUser, dwSize) = 0 then
      function = ""
   else
      function = left$(zUser, dwSize)
   end if

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'	getSpecialFolder
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function getSpecialFolder alias "getSpecialFolder" _
		( byval iWhichFolder as long ) export as string

	dim iReturn as long
	dim ppidl as long
	dim zReturn as asciiz * %MAX_PATH

	function = ""

	iReturn = shGetSpecialFolderLocation(0, iWhichFolder, ppidl)
	if iReturn = %NOERROR then
		iReturn = shGetPathFromIDList(byval ppidl, zReturn)
		if iReturn then
			function = zReturn
		end if
		coTaskMemFree ppidl
	end if


end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'	cleanFileName
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function cleanFileName(fileName as string) as string

	fileName = trim$(lcase$(fileName))
	fileName = retain$(fileName, any "abcdefghijklmnopqrstuvwxyz0123456789@#$()_-+.,"	)
	function = fileName

end function
