'
'  ff_pb_mtx.bas
'
'  Mutex wrappers for use in local locking
'  PUBLIC DOMAIN CODE
'  from Don Dickinson
'  don@greatwebdivide.com
'
'#include "ff_pb_secur.bas"

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  mtxLock
'
'  Returns %true if the mutex named sMutexName is able to be owned.
'  If not, it returns %false.
'
'  iTimeout_ms is the timeout value in milliseconds. It can be set
'  to %INFINITE to wait forever or 0 to return instantly.
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function mtxLock alias "mtxLock" _
      (byval sMutexName as string, byval iTimeout_ms as long ) _
       export as long

   dim hMutex as long

   '- Back slashes are not allowed so I replace
   '  with a ":"
   '
   replace "\" with ":" in sMutexName

   hMutex = createmutex(byval %null, 0, sMutexName + $nul)
   if hMutex = 0 then
      hMutex = openmutex(%MUTEX_ALL_ACCESS, 0, sMutexName + $nul)
   end if

   if hMutex = 0 then
      function = %false
   else
      select case waitforsingleobject(hMutex, iTimeout_ms)
         case %WAIT_ABANDONED, %WAIT_OBJECT_0
            function = %true
         case else      '%WAIT_FAILED, %WAIT_TIMEOUT
            function = %false
      end select
   end if

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  mtxLock2
'
'  Like mtxLock, but gets a security descriptor
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function mtxLock2 alias "mtxLock2" _
      (byval sMutexName as string, byval iTimeout_ms as long ) _
       export as long

   dim hMutex as long
   dim pSid as long
   dim pACL as long
   dim rSecurity as SECURITY_ATTRIBUTES
   dim rSecDesc as SECURITY_DESCRIPTOR

   '- Back slashes are not allowed so I replace
   '  with a ":"
   '
   replace "\" with ":" in sMutexName

   '- Get full rights
   if isNt() = %false then
      hMutex = createmutex(byval %null, 0, sMutexName + $nul)

   elseif getGlobalSecurity(rSecurity, rSecDesc, pSid, pACL) = %false then
      hMutex = createmutex(byval %null, 0, sMutexName + $nul)

   else
      hMutex = createmutex(rSecurity, 0, sMutexName + $nul)
   end if

   if hMutex = 0 then
      hMutex = openmutex(%MUTEX_ALL_ACCESS, 0, sMutexName + $nul)
   end if

   if hMutex = 0 then
      function = %false
   else
      select case waitforsingleobject(hMutex, iTimeout_ms)
         case %WAIT_ABANDONED, %WAIT_OBJECT_0
            function = %true
         case else      '%WAIT_FAILED, %WAIT_TIMEOUT
            function = %false
      end select
   end if

   if pSid then freesid pSid
   if pAcl then FreeMem pAcl

end function

'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'  mtxUnlock
'
'  Frees the mutex obtained with mtxLock - reference the mutex by name
'~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function mtxUnlock alias "mtxUnlock" _
      ( byval sMutexName as string ) export as long

   dim hMutex as long

   '- Just as in mtxLock, back slashes are not
   '  allowed so I replace with a ":"
   '
   replace "\" with ":" in sMutexName

   hMutex = openmutex(%MUTEX_ALL_ACCESS, 0, sMutexName + $nul)
   if hMutex then
      releasemutex hMutex
   end if

end function
